@@ -37,21 +37,12 @@
 #include <exchange-account.h>
 #include "exchange-component.h"
 
-#ifdef DEBUG
-#define LDAP_DEBUG
-#define LDAP_DEBUG_ADD
-#endif
-#include <ldap.h>
-#ifdef DEBUG
-#undef LDAP_DEBUG
-#endif
-
 #define d(x) x
 
 #include <sys/time.h>
 
 #include <libedataserver/e-sexp.h>
-#include <libedataserver/e-db3-utils.h>
+#include <libebackend/e-db3-utils.h>
 #include <libedataserver/e-data-server-util.h>
 #include <libebook/e-contact.h>
 
@@ -225,6 +216,15 @@ struct prop_info {
 
 static int num_prop_infos = sizeof(prop_info) / sizeof(prop_info[0]);
 
+static gboolean
+can_browse (EBookBackend *backend)
+{
+	return backend &&
+		e_book_backend_get_source (backend) &&
+		e_source_get_property (e_book_backend_get_source (backend), "can-browse") &&
+		strcmp (e_source_get_property (e_book_backend_get_source (backend), "can-browse"), "1") == 0;
+}
+
 static void
 book_view_notify_status (EDataBookView *view, const char *status)
 {
@@ -257,6 +257,7 @@ static GNOME_Evolution_Addressbook_CallStatus
 gal_connect (EBookBackendGAL *bl)
 {
 	EBookBackendGALPrivate *blpriv = bl->priv;
+	int ldap_error = 0;
 
 #ifdef DEBUG
 	{
@@ -277,9 +278,13 @@ gal_connect (EBookBackendGAL *bl)
 
 	g_object_ref (blpriv->gc);
 	g_mutex_lock (blpriv->ldap_lock);
-	blpriv->ldap = e2k_global_catalog_get_ldap (blpriv->gc, NULL);
+	blpriv->ldap = e2k_global_catalog_get_ldap (blpriv->gc, NULL, &ldap_error);
 	if (!blpriv->ldap) {
 		g_mutex_unlock (blpriv->ldap_lock);
+		d(printf ("%s: Cannot get ldap, error 0x%x (%s)\n", G_STRFUNC, ldap_error, ldap_err2string (ldap_error) ? ldap_err2string (ldap_error) : "Unknown error"));
+		if (ldap_error == LDAP_AUTH_METHOD_NOT_SUPPORTED)
+			return GNOME_Evolution_Addressbook_UnsupportedAuthenticationMethod;
+
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
 	}
 	g_mutex_unlock (blpriv->ldap_lock);
@@ -301,7 +306,7 @@ ldap_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, LDAP **ldap, int
 			book_view_notify_status (book_view, _("Reconnecting to LDAP server..."));
 
 		ldap_unbind (*ldap);
-		*ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
+		*ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL, NULL);
 		if (book_view)
 			book_view_notify_status (book_view, "");
 
@@ -322,7 +327,7 @@ gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 			book_view_notify_status (book_view, _("Reconnecting to LDAP server..."));
 		if (bl->priv->ldap)
 			ldap_unbind (bl->priv->ldap);
-		bl->priv->ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
+		bl->priv->ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL, NULL);
 		if (book_view)
 			book_view_notify_status (book_view, "");
 
@@ -1192,7 +1197,8 @@ build_query (EBookBackendGAL *bl, const char *query, char **ldap_query)
 	if (!r) {
 		/* Bad query or it isn't supported */
 		*ldap_query = NULL;
-		retval = GNOME_Evolution_Addressbook_QueryRefused;
+		e_sexp_unref (sexp);
+		return GNOME_Evolution_Addressbook_QueryRefused;
 	}
 
 	if (r->type == ESEXP_RES_STRING) {
@@ -1437,7 +1443,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 							char **email_values, **cn_values, **member_info;
 
 							if (!subldap) {
-								subldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
+								subldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL, NULL);
 							}
 							grpattrs[0] = "cn";
 							grpattrs[1] = "mail";
@@ -1650,7 +1656,10 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 		}
 		ldap_memfree (ldap_error_msg);
 
-		if (ldap_error == LDAP_TIMELIMIT_EXCEEDED)
+		if ((ldap_error == LDAP_TIMELIMIT_EXCEEDED || ldap_error == LDAP_SIZELIMIT_EXCEEDED) && can_browse ((EBookBackend *)bl))
+			/* do not complain when search limit exceeded for browsable LDAPs */
+			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_Success);
+		else if (ldap_error == LDAP_TIMELIMIT_EXCEEDED)
 			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_SearchTimeLimitExceeded);
 		else if (ldap_error == LDAP_SIZELIMIT_EXCEEDED)
 			e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_SearchSizeLimitExceeded);
@@ -1761,6 +1770,13 @@ start_book_view (EBookBackend  *backend,
 
 			status = build_query (bl, e_data_book_view_get_card_query (view),
 					      &ldap_query);
+
+			/* search for anything */
+			if (!ldap_query && (status == GNOME_Evolution_Addressbook_QueryRefused || status == GNOME_Evolution_Addressbook_Success) && can_browse ((EBookBackend *)bl)) {
+				ldap_query = g_strdup ("(mail=*)");
+				status = GNOME_Evolution_Addressbook_Success;
+			}
+
 			if (status != GNOME_Evolution_Addressbook_Success || !ldap_query) {
 				e_data_book_view_notify_complete (view, status);
 				if (ldap_query)
@@ -1821,7 +1837,14 @@ start_book_view (EBookBackend  *backend,
 
 			status = build_query (bl, e_data_book_view_get_card_query (view),
 					      &ldap_query);
-			printf("%s:%s: %s\n", G_STRLOC, G_STRFUNC, ldap_query);
+
+			/* search for anything */
+			if (!ldap_query &&  (status == GNOME_Evolution_Addressbook_QueryRefused || status == GNOME_Evolution_Addressbook_Success) && can_browse ((EBookBackend *)bl)) {
+				ldap_query = g_strdup ("(mail=*)");
+				status = GNOME_Evolution_Addressbook_Success;
+			}
+
+			printf("%s:%s: %s\n", G_STRLOC, G_STRFUNC, ldap_query ? ldap_query : "No ldap_query produced!");
 			if (status != GNOME_Evolution_Addressbook_Success || !ldap_query) {
 				e_data_book_view_notify_complete (view, status);
 				if (ldap_query)
@@ -1961,7 +1984,7 @@ tool_server_controls( LDAP *ld, LDAPControl *extra_c, int count )
 	}
 }
 
-#ifdef SUNLDAP
+#if defined (SUNLDAP) || defined (G_OS_WIN32)
 static struct berval *
 ber_dupbv( struct berval *dst, struct berval *src )
 {
@@ -2338,7 +2361,7 @@ ldap_cancel_op(void *key, void *value, void *data)
 	/* ignore errors, its only best effort? */
 	g_mutex_lock (bl->priv->ldap_lock);
 	if (bl->priv->ldap)
-		ldap_abandon_ext (bl->priv->ldap, op->id, NULL, NULL);
+		ldap_abandon (bl->priv->ldap, op->id);
 	g_mutex_unlock (bl->priv->ldap_lock);
 }
 
@@ -2632,7 +2655,10 @@ remove_gal (EBookBackend *backend, EDataBook *book, guint32 opid)
 static char *
 get_static_capabilities (EBookBackend *backend)
 {
-	return g_strdup("net");
+	if (can_browse (backend))
+		return g_strdup ("net,do-initial-query");
+	else
+		return g_strdup ("net");
 }
 
 /**
