@@ -46,6 +46,7 @@
 #include <camel/camel-stream-filter.h>
 #include <camel/camel-stream-mem.h>
 #include <camel/camel-folder-summary.h>
+#include <camel/camel-string-utils.h>
 
 static CamelOfflineFolderClass *parent_class = NULL;
 
@@ -70,8 +71,6 @@ static GPtrArray      *search_by_uids        (CamelFolder *folder,
 					      const char *expression,
 					      GPtrArray *uids,
 					      CamelException *ex);
-static void            search_free           (CamelFolder *folder,
-					      GPtrArray *uids);
 static void            transfer_messages_to  (CamelFolder *source,
 					      GPtrArray *uids,
 					      CamelFolder *dest,
@@ -101,7 +100,8 @@ class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->get_message = get_message;
 	camel_folder_class->search_by_expression = search_by_expression;
 	camel_folder_class->search_by_uids = search_by_uids;
-	camel_folder_class->search_free = search_free;
+	/* use the default function for the search_free */
+	/* camel_folder_class->search_free = search_free; */
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
 	camel_folder_class->refresh_info = refresh_info;
 	camel_folder_class->sync = exchange_sync;
@@ -229,7 +229,7 @@ append_message_data (CamelFolder *folder, GByteArray *message,
 
 	if (camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_APPEND_MESSAGE,
 			     CAMEL_STUB_ARG_FOLDER, folder->full_name,
-			     CAMEL_STUB_ARG_UINT32, camel_message_info_flags(info),
+			     CAMEL_STUB_ARG_UINT32, info? camel_message_info_flags(info): 0,
 			     CAMEL_STUB_ARG_STRING, subject,
 			     CAMEL_STUB_ARG_BYTEARRAY, message,
 			     CAMEL_STUB_ARG_RETURN,
@@ -464,27 +464,15 @@ search_by_expression (CamelFolder *folder, const char *expression,
 		      CamelException *ex)
 {
 	CamelFolderSearch *search;
-	GPtrArray *matches, *summary, *response;
-	int i;
+	GPtrArray *matches;
 
 	search = camel_exchange_search_new ();
 	camel_folder_search_set_folder (search, folder);
-	summary = camel_folder_get_summary (folder);
-	camel_folder_search_set_summary (search, summary);
 	matches = camel_folder_search_search (search, expression, NULL, ex);
-	camel_folder_free_summary (folder, summary);
-
-	if (matches) {
-		response = g_ptr_array_new ();
-		for (i = 0; i < matches->len; i++)
-			g_ptr_array_add (response, g_strdup (matches->pdata[i]));
-		camel_folder_search_free_result (search, matches);
-	} else
-		response = NULL;
 
 	camel_object_unref (CAMEL_OBJECT (search));
 
-	return response;
+	return matches;
 }
 
 static GPtrArray *
@@ -492,51 +480,16 @@ search_by_uids (CamelFolder *folder, const char *expression,
 		GPtrArray *uids, CamelException *ex)
 {
 	CamelFolderSearch *search;
-	GPtrArray *matches, *summary, *response;
-	int i;
-
-	summary = g_ptr_array_new();
-	for (i = 0; i < uids->len; i++) {
-		CamelMessageInfo *info;
-
-		info = camel_folder_get_message_info (folder, uids->pdata[i]);
-		if (info)
-			g_ptr_array_add (summary, info);
-	}
-
-	if (summary->len == 0)
-		return summary;
+	GPtrArray *matches;
 
 	search = camel_exchange_search_new ();
 	camel_folder_search_set_folder (search, folder);
-	camel_folder_search_set_summary (search, summary);
+	camel_folder_search_set_summary (search, uids);
 	matches = camel_folder_search_execute_expression (search, expression, ex);
 
-	if (matches) {
-		response = g_ptr_array_new ();
-		for (i = 0; i < matches->len; i++)
-			g_ptr_array_add (response, g_strdup (matches->pdata[i]));
-		camel_folder_search_free_result (search, matches);
-	} else
-		response = NULL;
-
-	for (i = 0; i < summary->len; i++)
-		camel_folder_free_message_info (folder, summary->pdata[i]);
-	g_ptr_array_free (summary, TRUE);
-
 	camel_object_unref (CAMEL_OBJECT (search));
 
-	return response;
-}
-
-static void
-search_free (CamelFolder *folder, GPtrArray *uids)
-{
-	int i;
-
-	for (i = 0; i < uids->len; i++)
-		g_free (uids->pdata[i]);
-	g_ptr_array_free (uids, TRUE);
+	return matches;
 }
 
 static void
@@ -787,7 +740,7 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	}
 	camel_object_unref (CAMEL_OBJECT (msg));
 
-	info->uid = g_strdup (uid);
+	info->uid = camel_pstring_strdup (uid);
 	einfo->info.flags = flags;
 	einfo->info.size = size;
 	einfo->href = g_strdup (href);
