@@ -27,11 +27,13 @@
 
 #include <glib/gi18n-lib.h>
 
+#include <camel/camel-session.h>
+#include <camel/camel-url.h>
+#include <libedataserver/e-data-server-util.h>
+
 #include "camel-exchange-store.h"
 #include "camel-exchange-folder.h"
 #include "camel-exchange-summary.h"
-#include <camel/camel-session.h>
-#include <camel/camel-url.h>
 
 #define SUBFOLDER_DIR_NAME     "subfolders"
 #define SUBFOLDER_DIR_NAME_LEN 10
@@ -320,29 +322,6 @@ get_name (CamelService *service, gboolean brief)
 
 #define EXCHANGE_STOREINFO_VERSION 1
 
-/* SURF : Picked this from gal/util/e-util.c */
-/* This only makes a filename safe for usage as a filename.
-	It still may have shell meta-characters in it. */
-static void
-e_filename_make_safe (gchar *string)
-{
-	gchar *p, *ts;
-	gunichar c;
-
-	g_return_if_fail (string != NULL);
-	p = string;
-
-	while(p && *p) {
-		c = g_utf8_get_char (p);
-		ts = p;
-		p = g_utf8_next_char (p);
-		if (!g_unichar_isprint(c) || ( c < 0xff && strchr (" /'\"`&();|<>$%{}!", c&0xff ))) {
-			while (ts<p)
-				*ts++ = '_';
-		}
-	}
-}
-
 static void
 camel_exchange_get_password (CamelService *service, CamelException *ex)
 {
@@ -376,11 +355,19 @@ camel_exchange_forget_password (CamelService *service, CamelException *ex)
 	}
 }
 
+static void
+update_camel_stub (gpointer folder_name, gpointer folder, gpointer user_data)
+{
+	CamelExchangeFolder *exch_folder = CAMEL_EXCHANGE_FOLDER (folder);
+	if (exch_folder)
+		exch_folder->stub = (CamelStub *)user_data;
+}
+
 static gboolean
 exchange_connect (CamelService *service, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (service);
-	char *real_user, *socket_path;
+	char *real_user, *socket_path, *dot_exchange_username, *user_at_host;
 	gchar *password = NULL;
 	guint32 connect_status;
 	gboolean online_mode = FALSE;
@@ -399,10 +386,15 @@ exchange_connect (CamelService *service, CamelException *ex)
 			real_user++;
 		else
 			real_user = service->url->user;
-		socket_path = g_strdup_printf ("/tmp/.exchange-%s/%s@%s",
-					       g_get_user_name (),
-					       real_user, service->url->host);
-		e_filename_make_safe (strchr (socket_path + 5, '/') + 1);
+		dot_exchange_username = g_strdup_printf (".exchange-%s", g_get_user_name ());
+		user_at_host = g_strdup_printf ("%s@%s", real_user, service->url->host);
+		e_filename_make_safe (user_at_host);
+		socket_path = g_build_filename (g_get_tmp_dir (),
+						dot_exchange_username,
+						user_at_host,
+						NULL);
+		g_free (dot_exchange_username);
+		g_free (user_at_host);
 
 		exch->stub = camel_stub_new (socket_path, _("Evolution Exchange backend process"), ex);
 		g_free (socket_path);
@@ -421,6 +413,8 @@ exchange_connect (CamelService *service, CamelException *ex)
 	if (online_mode) {
 		camel_exchange_get_password (service, ex);
 		if (camel_exception_is_set (ex)) {
+			camel_object_unref (exch->stub);
+			exch->stub = NULL;
 			g_mutex_unlock (exch->connect_lock);
 			return FALSE;
 		}
@@ -447,12 +441,16 @@ exchange_connect (CamelService *service, CamelException *ex)
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 				     _("Could not authenticate to server. "
 				       "(Password incorrect?)\n\n"));
+		camel_object_unref (exch->stub);
+		exch->stub = NULL;
 		g_mutex_unlock (exch->connect_lock);
 		return FALSE;
 	} else {
 		exch->stub_connected = TRUE;
 	}
 
+	g_hash_table_foreach (exch->folders, update_camel_stub, exch->stub);
+	
 	g_mutex_unlock (exch->connect_lock);
 
 	return TRUE;
@@ -461,6 +459,13 @@ exchange_connect (CamelService *service, CamelException *ex)
 static gboolean
 exchange_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
+
+	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (service);
+	
+	if (exch->stub) {
+		exch->stub = NULL;
+	}
+	
 	return TRUE;
 }
 
@@ -484,7 +489,7 @@ exchange_get_folder (CamelStore *store, const char *folder_name,
 	folder_dir = exchange_path_to_physical (exch->storage_path, folder_name);
 
 	if (!camel_exchange_store_connected (exch, ex)) {
-		if (!folder_dir || access (folder_dir, F_OK) != 0) {
+		if (!folder_dir || !g_file_test (folder_dir, G_FILE_TEST_IS_DIR)) {
 			g_free (folder_dir);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
 					      _("No such folder %s"), folder_name);
@@ -832,17 +837,75 @@ static void
 exchange_rename_folder (CamelStore *store, const char *old_name,
 			const char *new_name, CamelException *ex)
 {
+	GPtrArray *folders = NULL, *folder_names = NULL, *folder_uris = NULL;
+	GArray *unread_counts = NULL;
+	GArray *folder_flags = NULL;
+	CamelFolderInfo *info;
+	int i;
+	CamelRenameInfo reninfo;
+	CamelFolder *folder;
+
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 
 	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot rename folder in offline mode."));
 		return;
 	}
+	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_RENAME_FOLDER,
+			      CAMEL_STUB_ARG_STRING, old_name,
+			      CAMEL_STUB_ARG_STRING, new_name,
+			      CAMEL_STUB_ARG_RETURN,
+			      CAMEL_STUB_ARG_STRINGARRAY, &folder_names,
+			      CAMEL_STUB_ARG_STRINGARRAY, &folder_uris,
+			      CAMEL_STUB_ARG_UINT32ARRAY, &unread_counts,
+			      CAMEL_STUB_ARG_UINT32ARRAY, &folder_flags,
+			      CAMEL_STUB_ARG_END)) {
+		return;
+	}
+
+	if (!folder_names) {
+		/* This means the storage hasn't finished scanning yet.
+		 * We return NULL for now and will emit folder_created
+		 * events later.
+		 */
+		return;
+	}
+
+	folders = g_ptr_array_new ();
+	for (i = 0; i < folder_names->len; i++) {
+		info = make_folder_info (exch, folder_names->pdata[i],
+					 folder_uris->pdata[i],
+					 g_array_index (unread_counts, int, i),
+					 g_array_index (folder_flags, int, i));
+		if (info)
+			g_ptr_array_add (folders, info);
+	}
+	g_ptr_array_free (folder_names, TRUE);
+	g_ptr_array_free (folder_uris, TRUE);
+	g_array_free (unread_counts, TRUE);
+	g_array_free (folder_flags, TRUE);
+
+	info = camel_folder_info_build (folders, new_name, '/', TRUE);
+	
+	if (info)
+		info = postprocess_tree (info);
+	g_ptr_array_free (folders, TRUE);
+
+	reninfo.new = info;
+	reninfo.old_base = (char *)old_name;
+
+	g_mutex_lock (exch->folders_lock);
+	folder = g_hash_table_lookup (exch->folders, reninfo.old_base);
+	if (folder) {
+		g_hash_table_remove (exch->folders, reninfo.old_base);
+		camel_object_unref (CAMEL_OBJECT (folder));
+	}
+	g_mutex_unlock (exch->folders_lock);
+		
+	camel_object_trigger_event (CAMEL_OBJECT (exch),
+				    "folder_renamed", &reninfo);
+	camel_folder_info_free (reninfo.new);
 
-	camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_RENAME_FOLDER,
-			 CAMEL_STUB_ARG_FOLDER, old_name,
-			 CAMEL_STUB_ARG_FOLDER, new_name,
-			 CAMEL_STUB_ARG_END);
 }
 
 static void
@@ -1075,34 +1138,6 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		break;
 	}
 
-	case CAMEL_STUB_RETVAL_FOLDER_RENAMED:
-	{
-		char *new_name, *new_uri;
-		CamelFolder *folder;
-		CamelRenameInfo reninfo;
-
-		if (camel_stub_marshal_decode_folder (stub->status, &reninfo.old_base) == -1 ||
-		    camel_stub_marshal_decode_folder (stub->status, &new_name) == -1 ||
-		    camel_stub_marshal_decode_string (stub->status, &new_uri) == -1)
-			break;
-
-		reninfo.new = make_folder_info (exch, new_name, new_uri, -1, 0);
-
-		g_mutex_lock (exch->folders_lock);
-		folder = g_hash_table_lookup (exch->folders, reninfo.old_base);
-		if (folder) {
-			g_hash_table_remove (exch->folders, reninfo.old_base);
-			camel_object_unref (CAMEL_OBJECT (folder));
-		}
-		g_mutex_unlock (exch->folders_lock);
-
-		camel_object_trigger_event (CAMEL_OBJECT (exch),
-					    "folder_renamed", &reninfo);
-		camel_folder_info_free (reninfo.new);
-		g_free (reninfo.old_base);
-		break;
-	}
-
 	case CAMEL_STUB_RETVAL_FOLDER_SET_READONLY:
 	{
 		CamelFolder *folder;
